@@ -1,4 +1,4 @@
-cat("\nAdditional tests for predict():")
+cat(crayon::yellow("\nAdditional tests for predict():"))
 
 # test of bug corrected in version 2.6.0 (cf NEWS.Rd: "Bug in predict()'ing with an unusual combination of random effects...")
 npos <- c(11,16,14,2,6,1,1,4,10,22,7,1,0,0,1,6)
